package com.huimv.farm.damsubsidy.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.fasterxml.jackson.databind.ser.Serializers;
import com.huimv.farm.damsubsidy.common.token.TokenSign;
import com.huimv.farm.damsubsidy.common.utils.*;
import com.huimv.farm.damsubsidy.entity.*;
import com.huimv.farm.damsubsidy.entity.vo.BillBuyInfoVo;
import com.huimv.farm.damsubsidy.entity.BillLandingInspection;
import com.huimv.farm.damsubsidy.mapper.BaseAnimalMapper;
import com.huimv.farm.damsubsidy.mapper.BillBuyInfoMapper;
import com.huimv.farm.damsubsidy.mapper.BillLandingInspectionMapper;
import com.huimv.farm.damsubsidy.mapper.SysUserMapper;
import com.huimv.farm.damsubsidy.service.IBillBuyInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 交易信息 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-04-26
 */
@Service
public class BillBuyInfoServiceImpl extends ServiceImpl<BillBuyInfoMapper, BillBuyInfo> implements IBillBuyInfoService {
    @Resource
    private BaseAnimalMapper baseAnimalMapper;
    @Resource
    private BillBuyInfoMapper billBuyInfoMapper;
    @Resource
    private SysUserMapper sysUserMapper;
    @Resource
    private BillLandingInspectionMapper billLandingInspectionMapper;
    @Autowired
    private UploadImage uploadImage;
    @Override
    @Transactional(rollbackFor = RuntimeException.class)
    public synchronized Result addTransaction(HttpServletRequest httpServletRequest, MultipartFile buyImage1, MultipartFile buyImage2, MultipartFile buyImage3, MultipartFile buyImage4, MultipartFile buyImage5, MultipartFile quarantineImage, MultipartFile image1, MultipartFile image2, String inspectionNum, String cargoOwner, String ownerPhone, String animalType, String animalNumberUp, String useTo, String startTran, String reachTran, String seller, String sellerId, String buyer, String buyerId, String eartags, String buyerPhone, String sellerPhone) throws IOException {
        Long userId = TokenSign.getMemberIdByJwtToken(httpServletRequest);
        BillBuyInfo billBuyInfo1 = billBuyInfoMapper.selectOne(new QueryWrapper<BillBuyInfo>().eq("create_user", userId).eq("bill_buy_status",1));
        int total = Integer.parseInt(animalNumberUp);
        SysUser sysUser = sysUserMapper.selectOne(new QueryWrapper<SysUser>().eq("id", userId));

        QueryWrapper<BillLandingInspection> landingInspectionQueryWrapper = new QueryWrapper<>();
        landingInspectionQueryWrapper.eq("inspection_num", inspectionNum).notIn("inspection_sch",5,6);
        if (billLandingInspectionMapper.selectCount(landingInspectionQueryWrapper) != 0) {
            return new Result(10001, "存在相同检疫证号！", false);
        }
        if (inspectionNum.length() != 10) {
            return new Result(10001, "检疫证号应该为10位！", false);
        }
        if (!IntegerUtil.isInteger(animalNumberUp)){
            return new Result(10001, "头数应为整数！", false);
        }

        EartagResult eartagResult = EartagUtil.getEartagCollection(eartags);
        if (!eartagResult.getSuccess()){
            return new Result(10001,eartagResult.getMsg(),false);
        }
        if (eartagResult.getEartagList().size() != eartagResult.getEartagSet().size()) {
            return new Result(10001,"耳标号存在重复",false);
        }
        System.out.println("具体耳标号"+eartagResult.getEartagList());
        if (eartagResult.getEartagList().size() != total){
           return new Result(10001,"交易数量与实际耳标号数量不符!",false);
        }
        List<BaseAnimal> baseAnimals = new ArrayList<>();
        for (String eartagNo : eartagResult.getEartagList()) {
            BaseAnimal baseAnimal = baseAnimalMapper.selectOne(new QueryWrapper<BaseAnimal>().eq("earatg_no", eartagNo));
            if (baseAnimalMapper.selectCount(new QueryWrapper<BaseAnimal>().eq("earatg_no",eartagNo).eq("belong_to_user",sellerId).eq("is_loated",0).eq("is_die",0).eq("animal_status",1).eq("animal_type",animalType)) == 0) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return new Result(10001,"耳标号"+eartagNo+"不符合交易条件",false);
            }
            baseAnimal.setBelongToUser(Long.valueOf(buyerId));
            baseAnimal.setBuyerId(Long.valueOf(buyerId));
            baseAnimal.setSellerId(Long.valueOf(sellerId));
            baseAnimal.setBuyerName(buyer);
            baseAnimal.setSellerName(seller);
            baseAnimal.setAnimalStatus(2);
            baseAnimal.setInspectionNum(inspectionNum);
            baseAnimal.setBuyTime(DateTime.now());
            baseAnimals.add(baseAnimal);
        }
        for (BaseAnimal baseAnimal : baseAnimals) {
            if (baseAnimalMapper.updateById(baseAnimal) != 1) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return new Result(10001,"操作失败",false);
            }
        }
        String content3 = "";
        String content = uploadImage.getImageCom(image1);
        String content2 = uploadImage.getImageCom(image2);
        String imageCom = "";
        if (ObjectUtil.isNotEmpty(quarantineImage)){
            imageCom = uploadImage.getImageCom(quarantineImage);
        }
        content3 = getUrl(content3,buyImage1);
        content3 = getUrl(content3,buyImage2);
        content3 = getUrl(content3,buyImage3);
        content3 = getUrl(content3,buyImage4);
        content3 = getUrl(content3,buyImage5);
        BillLandingInspection billLandingInspection = billLandingInspectionMapper.selectOne(new QueryWrapper<BillLandingInspection>()
                .eq("create_user", buyerId)
                .eq("inspection_sch", 6));
        if (ObjectUtil.isNotEmpty(billLandingInspection)){
            billLandingInspection.setInspectionNum(inspectionNum);
            billLandingInspection.setInspectionName(buyer+"的产地检疫");
            billLandingInspection.setCargoOwner(cargoOwner);
            billLandingInspection.setOwnerPhone(ownerPhone);
            billLandingInspection.setAnimalType(Integer.parseInt(animalType));
            billLandingInspection.setAnimalNumberUp(animalNumberUp);
            billLandingInspection.setUseTo(useTo);
            billLandingInspection.setStartTran(startTran);
            billLandingInspection.setReachTran(reachTran);
            billLandingInspection.setEartags(eartags);
            billLandingInspection.setInspectionSch(4);
            billLandingInspection.setInspectionType(1);
            billLandingInspection.setReviewed(sysUser.getUserName());
            billLandingInspection.setReviewedTime(DateTime.now());
            billLandingInspection.setCreateUser(Long.valueOf(buyerId));
            billLandingInspection.setCreateTime(DateTime.now());
            billLandingInspectionMapper.updateById(billLandingInspection);
        }else {
            BillLandingInspection billLandingInspection1 = new BillLandingInspection();
            billLandingInspection1.setInspectionNum(inspectionNum);
            billLandingInspection1.setInspectionName(buyer+"的产地检疫");
            billLandingInspection1.setCargoOwner(cargoOwner);
            billLandingInspection1.setOwnerPhone(ownerPhone);
            billLandingInspection1.setAnimalType(Integer.parseInt(animalType));
            billLandingInspection1.setAnimalNumberUp(animalNumberUp);
            billLandingInspection1.setUseTo(useTo);
            billLandingInspection1.setStartTran(startTran);
            billLandingInspection1.setReachTran(reachTran);
            billLandingInspection1.setEartags(eartags);
            billLandingInspection1.setInspectionSch(4);
            billLandingInspection1.setInspectionType(1);
            billLandingInspection1.setReviewed(sysUser.getUserName());
            billLandingInspection1.setReviewedTime(DateTime.now());
            billLandingInspection1.setCreateUser(Long.valueOf(buyerId));
            billLandingInspection1.setCreateTime(DateTime.now());
            billLandingInspectionMapper.insert(billLandingInspection1);
        }

        if (ObjectUtil.isEmpty(billBuyInfo1)){
            BillBuyInfo billBuyInfo = new BillBuyInfo();
            billBuyInfo.setBuyName(buyer+"和"+seller+"的交易信息");
            billBuyInfo.setInspectionNum(inspectionNum);
            billBuyInfo.setBuyerName(buyer);
            billBuyInfo.setSellerName(seller);
            billBuyInfo.setSellerPhone(sellerPhone);
            billBuyInfo.setBuyerPhone(buyerPhone);
            billBuyInfo.setCreateUser(userId);
            billBuyInfo.setCreateTime(DateTime.now());
            billBuyInfo.setRealNumUp(animalNumberUp);
            billBuyInfo.setBuyPicUrl(content);
            billBuyInfo.setBuyerId(Long.valueOf(buyerId));
            billBuyInfo.setSellerId(Long.valueOf(sellerId));
            billBuyInfo.setPayPicUrl(content2);
            billBuyInfo.setBuyPayGoodsPicUrl(content3);
            billBuyInfo.setQuarantineCertificatePic(imageCom);
            billBuyInfo.setRealEartag(eartags);
            billBuyInfo.setBillBuyStatus(2);
            this.save(billBuyInfo);
        }else {
            billBuyInfo1.setBuyName(buyer+"和"+seller+"的交易信息");
            billBuyInfo1.setInspectionNum(inspectionNum);
            billBuyInfo1.setBuyerName(buyer);
            billBuyInfo1.setSellerName(seller);
            billBuyInfo1.setSellerPhone(sellerPhone);
            billBuyInfo1.setBuyerPhone(buyerPhone);
            billBuyInfo1.setCreateUser(userId);
            billBuyInfo1.setBuyerId(Long.valueOf(buyerId));
            billBuyInfo1.setSellerId(Long.valueOf(sellerId));
            billBuyInfo1.setCreateTime(DateTime.now());
            billBuyInfo1.setRealNumUp(animalNumberUp);
            billBuyInfo1.setBuyPicUrl(content);
            billBuyInfo1.setPayPicUrl(content2);
            billBuyInfo1.setQuarantineCertificatePic(imageCom);
            billBuyInfo1.setBuyPayGoodsPicUrl(content3);
            billBuyInfo1.setRealEartag(eartags);
            billBuyInfo1.setBillBuyStatus(2);
            this.updateById(billBuyInfo1);
        }

        return Result.SUCCESS();
    }

    @Override
    @Transactional(rollbackFor = RuntimeException.class)
    public Result deleteTransaction(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        BillBuyInfo billBuyInfo = billBuyInfoMapper.selectById(id);
        String inspectionNum = billBuyInfo.getInspectionNum();
        Long sellerId = billBuyInfo.getSellerId();
        List<String> eartagList = EartagUtil.getEartagCollection(billBuyInfo.getRealEartag()).getEartagList();
        for (String s : eartagList) {
            BaseAnimal baseAnimal = baseAnimalMapper.selectOne(new QueryWrapper<BaseAnimal>().eq("earatg_no", s));
            if (StringUtils.isBlank(baseAnimal.getInspectionNumBefore())){
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return new Result(10001,"此交易不符合删除条件,请联系管理员.",false);
            }
            baseAnimal.setAnimalStatus(1);
            baseAnimal.setSellerId(null);
            baseAnimal.setBuyerId(null);
            baseAnimal.setSellerName(null);
            baseAnimal.setBuyerName(null);
            baseAnimal.setBuyTime(null);
            baseAnimal.setBelongToUser(sellerId);
            baseAnimal.setInspectionNum(baseAnimal.getInspectionNumBefore());
            int update = baseAnimalMapper.updateById(baseAnimal);
            if (update != 1) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return new Result(10001,"删除失败,动物信息修改失败",false);
            }
        }

        int delete = billLandingInspectionMapper.delete(new QueryWrapper<BillLandingInspection>().eq("inspection_num", inspectionNum).eq("inspection_type", 1));
        if (delete != 1) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return new Result(10001,"删除失败,受影响的检疫证数：" + delete,false);
        }
        int delete1 = billBuyInfoMapper.deleteById(id);
        if (delete1 != 1) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return new Result(10001,"删除失败，受影响的交易数量：" + delete1,false);
        }
        return Result.SUCCESS();
    }

    public String getUrl(String content, MultipartFile image) throws IOException {
        if (ObjectUtil.isNotEmpty(image)){
            if ( !"".equals(content)){
                content = content +","+uploadImage.getImageCom(image);
            }else {
                content =   content +uploadImage.getImageCom(image);
            }
        }
        return content;
    }

    @Override
    @Transactional
    public Result saveTransaction(HttpServletRequest httpServletRequest,Map<String, String> paramsMap) {
        String inspectionNum = paramsMap.get("inspectionNum");
        String cargoOwner = paramsMap.get("cargoOwner");
        String ownerphone = paramsMap.get("ownerPhone");
        String animalType = paramsMap.get("animalType");
        String animalNumberUp = paramsMap.get("animalNumberUp");
        String useTo = paramsMap.get("useTo");
        String startTran = paramsMap.get("startTran");
        String reachTran = paramsMap.get("reachTran");
        String eartags = paramsMap.get("eartags");
        String sellerId = paramsMap.get("sellerId");
        String buyerId = paramsMap.get("buyerId");
        String seller = paramsMap.get("seller");
        String buyer = paramsMap.get("buyer");
        String sellerPhone = paramsMap.get("sellerPhone");
        String buyerPhone = paramsMap.get("buyerPhone");

        if (billLandingInspectionMapper.selectCount(new QueryWrapper<BillLandingInspection>().eq("inspection_num",inspectionNum)) != 0) {
            return new Result(10001,"该检疫证号已存在!",false);
        }

        BillLandingInspection billLandingInspection = billLandingInspectionMapper.selectOne(new QueryWrapper<BillLandingInspection>()
                .eq("create_user", buyerId)
                .eq("inspection_sch", 6));
        if (ObjectUtil.isNotEmpty(billLandingInspection)){
            billLandingInspection.setInspectionNum(inspectionNum);
            billLandingInspection.setInspectionName(buyer+"的产地检疫(保存)");
            billLandingInspection.setCargoOwner(cargoOwner);
            billLandingInspection.setOwnerPhone(ownerphone);
            billLandingInspection.setAnimalType(Integer.parseInt(animalType));
            billLandingInspection.setAnimalNumberUp(animalNumberUp);
            billLandingInspection.setInspectionType(1);
            billLandingInspection.setUseTo(useTo);
            billLandingInspection.setStartTran(startTran);
            billLandingInspection.setReachTran(reachTran);
            billLandingInspection.setEartags(eartags);
            billLandingInspectionMapper.updateById(billLandingInspection);
        }else {
            BillLandingInspection billLandingInspection1 = new BillLandingInspection();
            billLandingInspection1.setInspectionNum(inspectionNum);
            billLandingInspection1.setInspectionName(buyer+"的产地检疫(保存)");
            billLandingInspection1.setCargoOwner(cargoOwner);
            billLandingInspection1.setOwnerPhone(ownerphone);
            billLandingInspection1.setAnimalType(Integer.parseInt(animalType));
            billLandingInspection1.setAnimalNumberUp(animalNumberUp);
            billLandingInspection1.setInspectionType(1);
            billLandingInspection1.setUseTo(useTo);
            billLandingInspection1.setStartTran(startTran);
            billLandingInspection1.setReachTran(reachTran);
            billLandingInspection1.setEartags(eartags);
            billLandingInspection1.setCreateTime(DateTime.now());
            billLandingInspection1.setCreateUser(Long.valueOf(buyerId));
            billLandingInspection1.setInspectionSch(6);
            billLandingInspectionMapper.insert(billLandingInspection1);
        }

        BillBuyInfo billBuyInfo1 = billBuyInfoMapper.selectOne(new QueryWrapper<BillBuyInfo>().eq("create_user", TokenSign.getMemberIdByJwtToken(httpServletRequest)).eq("bill_buy_status",1));
        if (ObjectUtil.isEmpty(billBuyInfo1)){
            BillBuyInfo billBuyInfo = new BillBuyInfo();
            billBuyInfo.setInspectionNum(inspectionNum);
            billBuyInfo.setBuyName(buyer+"和"+seller+"的交易信息");
            billBuyInfo.setBuyerId(Long.valueOf(buyerId));
            billBuyInfo.setSellerId(Long.valueOf(sellerId));
            billBuyInfo.setBuyerName(buyer);
            billBuyInfo.setBuyerPhone(buyerPhone);
            billBuyInfo.setSellerName(seller);
            billBuyInfo.setCreateUser(TokenSign.getMemberIdByJwtToken(httpServletRequest));
            billBuyInfo.setCreateTime(DateTime.now());
            billBuyInfo.setBillBuyStatus(1);
            this.save(billBuyInfo);
        }else {
            billBuyInfo1.setBuyName(buyer+"和"+seller+"的交易信息");
            billBuyInfo1.setInspectionNum(inspectionNum);
            billBuyInfo1.setBuyerId(Long.valueOf(buyerId));
            billBuyInfo1.setSellerId(Long.valueOf(sellerId));
            billBuyInfo1.setBuyerName(buyer);
            billBuyInfo1.setBuyerPhone(buyerPhone);
            billBuyInfo1.setSellerName(seller);
            billBuyInfo1.setSellerPhone(sellerPhone);
            billBuyInfo1.setCreateUser(TokenSign.getMemberIdByJwtToken(httpServletRequest));
            billBuyInfo1.setCreateTime(DateTime.now());
            billBuyInfo1.setBillBuyStatus(1);
            this.updateById(billBuyInfo1);
        }
        return Result.SUCCESS();
    }

    @Override
    public Result getLast(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        BillBuyInfoVo billBuyInfo = billBuyInfoMapper.getLast(TokenSign.getMemberIdByJwtToken(httpServletRequest));
        if (ObjectUtil.isEmpty(billBuyInfo)){
            return new Result(ResultCode.SUCCESS,null);
        }else {
            return new Result(ResultCode.SUCCESS,billBuyInfo);
        }
    }

    @Override
    public Result listDetail(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        BillBuyInfoVo billBuyInfo= billBuyInfoMapper.listDetail(id);
        return new Result(ResultCode.SUCCESS,billBuyInfo);
    }
}
