package com.huimv.farm.damsubsidy.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.huimv.farm.damsubsidy.common.token.TokenSign;
import com.huimv.farm.damsubsidy.common.utils.Result;
import com.huimv.farm.damsubsidy.common.utils.ResultCode;
import com.huimv.farm.damsubsidy.entity.AreaAll;
import com.huimv.farm.damsubsidy.entity.BillBuyInfo;
import com.huimv.farm.damsubsidy.entity.BillSubsidy;
import com.huimv.farm.damsubsidy.entity.BillSubsidyBatch;
import com.huimv.farm.damsubsidy.entity.vo.BillSubsidyDto;
import com.huimv.farm.damsubsidy.entity.vo.BillSubsidyEnterpriseVo;
import com.huimv.farm.damsubsidy.entity.vo.BillSubsidyTotalVo;
import com.huimv.farm.damsubsidy.mapper.AreaAllMapper;
import com.huimv.farm.damsubsidy.mapper.BillBuyInfoMapper;
import com.huimv.farm.damsubsidy.mapper.BillSubsidyBatchMapper;
import com.huimv.farm.damsubsidy.mapper.BillSubsidyMapper;
import com.huimv.farm.damsubsidy.service.IBillSubsidyBatchService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.farm.test.Print3;
import com.spire.ms.System.Collections.ArrayList;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 补贴批次表 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-06-19
 */
@Service
public class BillSubsidyBatchServiceImpl extends ServiceImpl<BillSubsidyBatchMapper, BillSubsidyBatch> implements IBillSubsidyBatchService {

    @Autowired
    private BillSubsidyBatchMapper billSubsidyMapper;
    @Autowired
    private BillSubsidyMapper subsidyMapper;
    @Autowired
    private AreaAllMapper areaAllMapper;
    @Autowired
    private BillBuyInfoMapper buyInfoMapper;

    @Override
    public Result add(HttpServletRequest httpServletRequest, BillSubsidyBatch billSubsidyBatch) {
        Long id = TokenSign.getMemberIdByJwtToken(httpServletRequest);
        Date date = new Date();
        billSubsidyBatch.setCreateTime(date);
        billSubsidyBatch.setCreateUser(id.toString());
        String subsidyIds = billSubsidyBatch.getSubsidyIds();
        String[] split = subsidyIds.split(",");
        for (String s : split) {
            QueryWrapper<BillSubsidy> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("id", s);
            BillSubsidy billSubsidy1 = subsidyMapper.selectOne(queryWrapper);
            billSubsidy1.setIsBatch(1);
            subsidyMapper.updateById(billSubsidy1);
        }
        billSubsidyMapper.insert(billSubsidyBatch);
        return new Result(10000, "添加成功！", true);
    }

    @Override
    public Result list(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String name = paramsMap.get("name");//支付批次名称
        QueryWrapper<BillSubsidyBatch> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(StringUtils.isNotBlank("batch_name"), "batch_name", name);
        List<BillSubsidyBatch> billSubsidyBatches = billSubsidyMapper.selectList(queryWrapper);
        return new Result(ResultCode.SUCCESS, billSubsidyBatches);
    }

    @Override
    public Result listById(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String id = paramsMap.get("id");//批次表中的id
        String name = paramsMap.get("name");
        QueryWrapper<BillSubsidyBatch> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", id);
        BillSubsidyBatch batch = billSubsidyMapper.selectOne(queryWrapper);
        String subsidyIds = batch.getSubsidyIds();
        String[] split = subsidyIds.split(",");
        JSONObject jsonObject = new JSONObject();
        List<BillSubsidyDto> list = new ArrayList();
        for (String s : split) {
            QueryWrapper<BillSubsidy> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.eq("id",s).eq(StringUtils.isNotBlank(name), "farmer_name", name);
            BillSubsidy billSubsidy = subsidyMapper.selectOne(queryWrapper1);
            BillSubsidyDto dto = new BillSubsidyDto();
            BeanUtil.copyProperties(billSubsidy, dto);
            QueryWrapper<AreaAll> queryWrapper2 = new QueryWrapper<>();
            String detailedAdressCode = dto.getDetailedAdressCode();
            String[] split1 = detailedAdressCode.split(",");
            queryWrapper2.eq("id",split1[2]);
            AreaAll areaAll = areaAllMapper.selectOne(queryWrapper2);
            dto.setCountry(areaAll.getName());
            list.add(dto);
        }
        jsonObject.put("ids", subsidyIds);
        jsonObject.put("batchName", batch.getBatchName());
        jsonObject.put("list", list);
        Date createTime = batch.getCreateTime();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        jsonObject.put("time", sdf.format(createTime));
        return new Result(ResultCode.SUCCESS, jsonObject);
    }

    @Override
    public void printHua(Map<String, String> paramsMap) {
        String ids = paramsMap.get("ids");
        String batchName = paramsMap.get("batchName");//批次
        String date = paramsMap.get("date");//时间

        String[] split = ids.split(",");
        List<BillSubsidyDto> subsidyList = new ArrayList();
        for (String s : split) {
            BillSubsidyDto subsidyDto = new BillSubsidyDto();
            QueryWrapper<BillSubsidy> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("id", s);
            BillSubsidy billSubsidy = subsidyMapper.selectOne(queryWrapper);
            BeanUtil.copyProperties(billSubsidy, subsidyDto);
            String num = subsidyDto.getInspectionNum();
            QueryWrapper<BillBuyInfo> queryWrapper2 = new QueryWrapper<>();
            queryWrapper2.eq("inspection_num", num);
            BillBuyInfo billBuyInfo = buyInfoMapper.selectOne(queryWrapper2);

            String detailedAdressCode = billSubsidy.getDetailedAdressCode();
            String[] split1 = detailedAdressCode.split(",");
            QueryWrapper<AreaAll> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.eq("id",  split1[2]);
            AreaAll areaAll = areaAllMapper.selectOne(queryWrapper1);
            subsidyDto.setSellerName(billBuyInfo.getSellerName());
            subsidyDto.setCountry(areaAll.getName());
            System.out.println(subsidyDto);
            subsidyList.add(subsidyDto);
        }
        try {
            Print3.print2(subsidyList, batchName, date);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    @Override
    public void printCount(Map<String, String> paramsMap) {
        String ids = paramsMap.get("ids");
        String batchName = paramsMap.get("batchName");//批次
        String date = paramsMap.get("date");//时间
        ids = "("+ids+")";

        List<BillSubsidyTotalVo> billSubsidyTotalVos = subsidyMapper.listTotal(ids);
        try {
            Print3.print3(billSubsidyTotalVos,batchName,date);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void printDetail(Map<String, String> paramsMap) {
        String ids = paramsMap.get("ids");
        String batchName = paramsMap.get("batchName");//批次
        String date = paramsMap.get("date");//时间
        List<BillSubsidyEnterpriseVo> billSubsidyEnterpriseVos = subsidyMapper.listEnterprise("("+ids+")");
        try {
            Print3.print4(billSubsidyEnterpriseVos,batchName,date);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
